 /*
 *  Access IO Test Program 
 *
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <pthread.h>   // for nanosleep() call

#include "aioUsbApi.h"


int   devIdx;
char  anyKey;
int   ctlC;
aioDeviceInfo aioDevices;

#define MAX_ROW 25
#define MAX_COL 70

struct timespec      bulkPollTime;

void
catchInterrupt (int signum) 
{
    ctlC = 1;
}


int 
validateIndex(int devIdx)
{
  int ret;

  ret = AIO_UsbValidateDeviceIndex(devIdx);
  if (ret > ERROR_SUCCESS)
  {
     printf("Invalid device (%d); Do a GetDevices() to see valid devices  \n",devIdx);
  }
  return  ret;
}


void
listAIODevices()
{
  int i;

  if (aioDevices.numAccesDevs == 0)
    printf("\n\nNo AIO Devices Found\n");
  else
  {
    for (i=0; i<aioDevices.numAccesDevs; i++)
    {
      printf("Found AIO Device Number-%d \n",(unsigned int)aioDevices.aioDevList[i].devIdx);
      printf(" Product ID =0x%x\n",aioDevices.aioDevList[i].productId);
    } 
  }
}

/******** 
 errStr[ERROR_SUCCESS-1]                 =   "ERROR_SUCCESS";
 errStr[ERROR_NO_AIO_DEVS_FOUND-1]       =   "ERROR_NO_DEVS_FOUND"; 
 errStr[ERROR_FILE_NOT_FOUND-1]	         =   "ERROR_FILE_NOT_FOUND";	
 errStr[ERROR_DUP_NAME-1]                =   "ERROR_DUP_NAME";
 errStr[ERROR_DEV_NOT_EXIST-1]           =   "ERROR_DEV_DOES_NOT_EXIST";
 errStr[ERROR_DEVICE_REMOVED-1]          =   "ERROR_DEV_REMOVED";
 errStr[ERROR_INVALID_DEV_IDX-1]         =   "ERROR_INVALID_DEV_IDX";
 errStr[ERROR_CANNOT_OPEN_DEVICE-1]      =   "ERROR_CANNOT_OPEN_DEV";
 errStr[ERROR_USB_CONTROL_MSG_FAILED-1]  =   "ERROR_USB_CONTROL_MSG_FAILED";
 errStr[ERROR_INVALID_DATA-1]            =   "ERROR_INVALID_DATA";
 errStr[ERROR_UNSUPPORTED_DEVICE-1]      =   "ERROR_UNSUPPPORTED_DEV";
 errStr[ERROR_FUNCTION_NOT_SUPPORTED-1]  =   "ERROR_FUNCTION_NOT_SUPPORTED";
 errStr[ERROR_INVALID_INPUT_PARAM-1]     =   "ERROR_INVALID_INPUT_PARAM";
********/



int 
getFunction()
{
  

   int funcNum; 
   
   
  

//   ret = AIO_Usb_GetDevices(&aioDevices);
//   listDevices();
//   if (ret == ERROR_NO_AIO_DEVS_FOUND)
//   {
//    printf("NO Access/IO Devices found.  \n");
 // }    


   ctlC = 0;

   printf ("\n\n\n >>> Press the ANY key then CR to continue  ");
   scanf ("%s",&anyKey);
   printf ("\33[2J\n"); // clear screen
   printf ("\n\n\n\t\t\t------------------------------\n"); 
   printf ("\t\t\tACCES/IO Linux USB API Tester \n");
   printf ("\t\t\t------------------------------\n\n\n"); 

   printf ("\n\nNOTE : The USB-AI16-16A is the only board supported by the API currently\n\n\n");
    



   printf ("0) Take Analog to Digital Samples \n\n\n");

   printf ("1)GetDevices             11)Get ADC Config     21)Set Mode,Load Cntr         99)Exit\n");
   printf ("2)QueryDeviceInfo        12)Set ADC Config     22)Set Mode,Load Read Cntr           \n");
   printf ("3)ClearDevices           13)Set all Gain Codes 23)Read Cntr                         \n"); 
   printf ("4)Configure DIO Bytes    14)Set 1 Gain Code    24)Get 1 AtoD Sample                 \n"); 
   printf ("5)Write to 1 DIO pin     15)Set Trigger Mode                                        \n"); 
   printf ("6)Write to 1 DIO Byte    16)Set Data Rate                                           \n"); 
   printf ("7)Write to all DIO Bytes 17)Write to EEPROM                                         \n");
   printf ("8)Read 1 DUI Byte        18)Read from EEPROM                                        \n"); 
   printf ("9)Read All DIO Bytes     19)Set Cntr Mode                                           \n");
   printf ("10)Set Calibration       20)Load Cntr                                               \n");


   printf ("\n\n Enter function to call (99 to exit) : ");
   scanf ("%d",&funcNum);
   printf("\n\n");
     return funcNum;

}

char *
errorNumAsString(char *pData, int error)
{
  return ("Nothing here \n");
}

int main()
{
   
  
  int ret; 


 // use the first device found
 //devIdx = aioDevices.aioDevList[0].devIdx;

 // This ABSOLUTELY POSITIVELY MUST be called BEFORE any other function in the API
 // is called or bad things will happen (Segmentation Violation)
 //
 AIO_Init();

 // This should called BEFORE any other function in the API
 // to populate the list of Acces devices
 ret = AIO_Usb_GetDevices(&aioDevices);

 // use the first device found
 devIdx = aioDevices.aioDevList[0].devIdx;


 while (1)
 {
  switch (getFunction() ) 
  {
   case 99:
    return 0; 

   case 1:
   {
     int ret;
     
      ret = AIO_Usb_GetDevices(&aioDevices);
      if (ret > ERROR_SUCCESS)
      {
       printf("AIO_Usb_GetDevices failed! ret=%d \n",ret);
       break;
      }
     AIO_UsbListDevices();

     //scanf ("%d",(unsigned int *)&devIdx);
     printf ("\n\n Enter Device Idx : ");
     scanf ("%d",(unsigned int *)&devIdx);
  
     ret = validateIndex(devIdx);
     if (ret > ERROR_SUCCESS)
     {
       printf("\n\nDBG>> Invalid Device Index \n"); 
       break;
     }

      break;
    }

   case 2:
   {
        unsigned long	ProdID;
        unsigned long	NameSize;
        char            Name[100];
        unsigned long	DIOBytes;
        unsigned long	Counters;
        unsigned long	ret;

     ret = validateIndex(devIdx);
     if (ret > ERROR_SUCCESS)
       break;

     ret =  AIO_Usb_QueryDeviceInfo(
                        devIdx,
                        &ProdID,
                        &NameSize,
                        Name,
                        &DIOBytes,
                        &Counters);

    if (ret == ERROR_SUCCESS)
    {
      printf ("Dev Index = %d \n",(unsigned int)devIdx);
      printf ("Prod ID   = 0x%0x \n",(unsigned int)ProdID);
      printf ("Name Size = %d \n", (unsigned int)NameSize);
      printf ("Name      = %s \n", Name);
      printf ("DIO Bytes = %d \n", (unsigned int)DIOBytes); 
      printf ("Counters  = %d \n", (unsigned int)Counters); 
     break;
    }
   else
    printf("\n\nDBG>> AIO_UsbQueryDeviceInfo Failed dev=0x%0x\n\n",(unsigned int)devIdx);
   }
   break;

   case 3:
   {
    int ret;

     ret = validateIndex(devIdx);
     if (ret > ERROR_SUCCESS)
       break;

     AIO_Usb_ClearDevices();
     break;
   }
   case 4:
   {

     int            ret;
     unsigned char  data[4];
     int            triState; 
     unsigned char  mask; 
     int            tmp;
     unsigned char  pins0_7;
     unsigned char  pins8_15;
     int            p0_7Input;
     int            p8_15Input;

      //ret = validateIndex(devIdx);

      //if (ret > ERROR_SUCCESS)
      // break;

      //printf("\n\n");

      // AI 16 only has 16 pins 


       printf("Pins 0 - 7 {0=INput; 1=OUTput}                                  :  ");
       scanf("%x",&tmp);
       pins0_7 = tmp;
       p0_7Input = tmp;
       printf("Pins 8 - 15 {0=INput; 1=OUTput}                                 :  ");
       scanf("%x",&tmp);
       pins8_15 = tmp;
       p8_15Input = tmp;
     
      mask = 0;
      mask = pins8_15 << 1;;  
      mask = mask | pins0_7; 


      printf("Enter Tristate (0=off; 1=on}                                    :  ");
      scanf("%x",&tmp);
      triState= tmp;


      //first 2 bytes unused for AI-16
      //printf("Enter output byte  0 (hex)                                      :  ");
      //pscanf("%x",&tmp);
      ///pata[0] = tmp;

      //printf("Enter output byte  1 (hex)                                      :  ");
      //scanf("%x",&tmp);
      //data[1] = tmp;

      if (p0_7Input == 1)
      {
        printf("Enter output byte  0 (hex)                                      :  ");
        scanf("%x",&tmp);
        data[0] = tmp;
      }

      if (p8_15Input == 1)
      {
        printf("Enter output byte  1 (hex)                                      :  ");
        scanf("%x",&tmp);
        data[1] = tmp;
      }
      // ignored for AI16
      data[2] = 0;
      data[3] = 0;
      
      ret =   AIO_Usb_DIO_Configure (devIdx,
                                     triState,
                                     &mask,
                                     data);

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nDIO_Configure Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }

/*******
      ret =   AIO_Usb_DIO_ReadAll (devIdx,
                                   &data1[0]); 

      if (ret > ERROR_SUCCESS)
      {
        printf ("ReadAll() to readback bytes failed;  dev=0x%0x err=%d",(unsigned int)devIdx,ret);
        break;
      } 
 
      
//      printf("byte 0 = 0x%x\n",readBackData[0]);
//      printf("byte 1 = 0x%x\n",readBackData[1]);

      // AI 16 only has 16 pins

//      printf("\n PINs 0-7 : = 0x%x\n",readBackData[2]);
//      printf("\n PINs 8-15: = 0x%x\n",readBackData[3]);
   
     anyKey = getchar();
****/


     break;
   }
   case 5:
   {
      int            bitIdx; 
      int            setBit;
      char           data[4];
      int            ret;


      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      printf("Enter bitIndex                         : ");
      scanf("%d",(unsigned int *)&bitIdx);

      printf("Enter (0) to clear bit , (1) to set bit: ");
      scanf("%d",&setBit);

      ret = AIO_Usb_Write1(devIdx,
                           bitIdx,
                           setBit); 


      if (ret > ERROR_SUCCESS)
      {
        printf ("\n\nDIO_Write1 : Failed dev=0x%0x err=%d  \n\n",(unsigned int)devIdx,ret);
        break;
      }


       ret =   AIO_Usb_DIO_ReadAll (devIdx,
                                   (unsigned char *)&data[0]); 


       if (ret > ERROR_SUCCESS)
       {
        printf ("call to ReadAll() AFTER call to Write1 failed;  dev=0x%0x err=%d",(unsigned int)devIdx,ret);
        break;
       } 

      // Only first 2 valid for USB-AI16-16A> 
      // printf("\nDIO pins AFTER call to Write1 : \n");

//      printf("byte 0 = 0x%x\n",data[0]);
//      printf("byte 1 = 0x%x\n",data[1]);
   
//      printf("\n PINs 0-7 : = 0x%x\n",data[2]);
//      printf("\n PINs 8-15: = 0x%x\n",data[3]);

     break;
   }
   case 6:
   {
      unsigned long  byteIdx; 
      char           data[4];
      int            ret;
      char           byte;


      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      if (ret > ERROR_SUCCESS)
       break;
      printf("Enter byte Index      :  ");
      scanf("%x",(unsigned int *)&byteIdx);

      printf("Enter byte value (hex):  ");
      scanf("%x",(unsigned int *)&byte);

       ret = AIO_Usb_Write8(devIdx,
                            byteIdx,
                            byte);


      if (ret > ERROR_SUCCESS)
      {
        printf ("\n\nDIO_Write8 : Failed dev=0x%0x err=%d  \n\n",(unsigned int)devIdx,ret);
        break;
      }

       ret =   AIO_Usb_DIO_ReadAll (devIdx,
                                   (unsigned char *)&data[0]); 


      if (ret > ERROR_SUCCESS)
      {
        printf (" ReadAll() AFTER call to Write1 failed;  dev=0x%0x err=%d",(unsigned int)devIdx,ret);
        break;
      } 

      //printf("\n PINs 0-7 : = 0x%x\n",data[2]);
      //printf("\n PINs 0-15: = 0x%x\n",data[3]);

      //printf("\n PINs 0- 7: = 0x%x\n",data[0]);
      //printf("\n PINs 0-15: = 0x%x\n",data[1]);


 
     break;
   }
   case 7:
   {
      unsigned char           data[4];
      
      int            ret;
      int             tmp;

      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      // first 2 bytes unused for AI 16
      //printf("Enter output byte  2 (hex) :  ");
      //scanf("%x",&tmp);
      //data[0] = tmp;

      //printf("Enter output byte  3 (hex) :  ");
      //scanf("%x",&tmp);
      //data[1] = tmp;


      printf("Enter output byte  0 (hex) :  ");
      scanf("%x",&tmp);
      data[0] = tmp;

      printf("Enter output byte  1 (hex) :  ");
      scanf("%x",&tmp);
      data[1] = tmp;



      ret = AIO_Usb_WriteAll(devIdx,
                             (unsigned char *)&data[0]);


      if (ret > ERROR_SUCCESS)
      {
        printf ("\n\nDIO_WriteAll : Failed dev=0x%0x err=%d  \n\n",(unsigned int)devIdx,ret);
        break;
      }


     break;
   }
   case 8:
   {
      unsigned char  byte;
      unsigned int   byteIdx;
      int            ret;

      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      printf("Enter byteIdx    :  ");
      scanf("%d",(unsigned int*)&byteIdx);

     ret = AIO_Usb_DIO_Read8 (devIdx,
                              byteIdx,
                              &byte);

      if (byteIdx == 0)
         printf(" \nPINs 0-7 : 0x%x\n",(unsigned char)byte);
      else
         printf(" \nPINs 0-15: 0x%x\n",(unsigned char)byte);

     break;
     
   }
   case 9:
   {
     int            ret;
     unsigned char           data[4];
    
 

      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      ret =   AIO_Usb_DIO_ReadAll (devIdx,
                                   (unsigned char *)&data[0]); 

      if (ret > ERROR_SUCCESS)
      {
        printf ("\n\nReadAll Failed dev=0x%0x err=%d  \n\n",(unsigned int)devIdx,ret);
        break;
      }

      // first 2 bytes unused for AI 16
      //printf("\n PINs 0- 7: = 0x%x\n",data[0]);
      //printf("\n PINs 0-15: = 0x%x\n",data[1]);

      printf("\n PINs 0- 7: = 0x%x\n",data[0]);
      printf("\n PINs 0-15: = 0x%x\n",data[1]);

     break;
   }
   case 10:
   {

     int            ret;
     char           calFname[40];

      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      printf ("\n\n Enter Calibration Filename, or ':NONE:' for default : ");

     scanf ("%s",calFname);

     ret=  validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;
     
     ret =   AIO_Usb_ADC_SetCal(devIdx,
                                calFname);
     
     if (ret > ERROR_SUCCESS)
     {
       printf("ADC_SetCal failed err=%d \n",ret);
       break;
     }
     break;
   }

  
   case 11:
   {
     int            i;
     int            ret;
     
     unsigned char  cfgBuf2[20];
     unsigned long  cfgBufSize;


      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      ret =  AIO_Usb_ADC_GetConfig (devIdx,
                                    &cfgBuf2[0],
                                    &cfgBufSize); 
     
     if (ret > ERROR_SUCCESS)
     {
      printf ("\nGetConfig Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }
   
     printf("\t     Note that 8 is added to chan 0-7 for differential mode.\n"); 
     for (i=0; i < 16; i++)
     {
       printf("Channel %d Gain Code  = %d\n",i,cfgBuf2[i]);
     } 

     printf("Calibration Code      =  %d \n",cfgBuf2[16]);
     printf("OverSampling          =  %d \n",cfgBuf2[19]);

     printf("Start Chan            =  %d \n",(cfgBuf2[18] & 0x0F ) );
     printf("End  Chan             =  %d \n",(cfgBuf2[18] >> 4) );

     printf("CTR0 EXT              =  %d \n",((cfgBuf2[17] & 0x10)  >> 4 ) );
     printf("Falling Edge          =  %d \n",((cfgBuf2[17] & 0x08)  >> 3 ) );
     printf("Scan                  =  %d \n",((cfgBuf2[17] & 0x07)  >> 2) );
     printf("External Trigger      =  %d \n",((cfgBuf2[17] & 0x02)  >> 1 ) );
     printf("Timer Trigger         =  %d \n",((cfgBuf2[17] & 0x01)  >> 0 ) );

     break;
   }

   case 12:
   {

     char           trigger;
     char           strtEndChan;
     char           startChan;
     char           endChan;
     int            ret;
     int            tmp;
     unsigned char  cfgBuf1[20];
     unsigned long  cfgBufSize;

      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      printf("Enter Chan-0 Gain code  (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[0] = tmp; 
      printf("Enter Chan-1 Gain code  (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[1] = tmp; 
      printf("Enter Chan-2 Gain code  (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[2] = tmp; 
      printf("Enter Chan-3 Gain code  (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[3] = tmp; 
      printf("Enter Chan-4 Gain code  (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[4] = tmp; 
      printf("Enter Chan-5 Gain code  (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[5] = tmp; 
      printf("Enter Chan-6 Gain code  (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[6] = tmp; 
      printf("Enter Chan-7 Gain code  (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[7] = tmp; 
      printf("Enter Chan-8 Gain code  (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[8] = tmp; 
      printf("Enter Chan-9 Gain code  (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[9] = tmp; 
      printf("Enter Chan-10 Gain code (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[10] = tmp; 
      printf("Enter Chan-11 Gain code (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[11] = tmp; 
      printf("Enter Chan-12 Gain code (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[12] = tmp; 
      printf("Enter Chan-13 Gain code (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[13] = tmp; 
      printf("Enter Chan-14 Gain code (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[14] = tmp; 
      printf("Enter Chan-15 Gain code (0 - 7)  : ");
      scanf("%d",&tmp);
      cfgBuf1[15] = tmp; 

      printf("Enter Calibration code (0,1,3)   : ");
      scanf("%d",&tmp);
      cfgBuf1[16] = tmp; 

      printf("Enter CTR0 EXT (0/1)             : ");
      scanf("%d",&tmp);
      trigger = trigger | (tmp << 4);

      printf("Enter Falling Edge (0/1)         : ");
      scanf("%d",&tmp);
      trigger = trigger | (tmp << 3);

      printf("Enter Scan (0/1)                 : ");
      scanf("%d",&tmp);
      trigger = trigger | (tmp << 2);

      printf("Enter External Trigger (0/1)     : ");
      scanf("%d",&tmp);
      trigger = trigger | (tmp << 1);

      printf("Enter Timer  Trigger (0/1)       : ");
      scanf("%x",&tmp);
      trigger = trigger | tmp;
      cfgBuf1[17] = trigger; 


      strtEndChan = 0;
      printf("Enter End Channel (0-15)         : ");
      scanf("%d",&tmp);
      startChan = tmp;
      strtEndChan = strtEndChan  | (tmp << 4);

      printf("Enter Start Channel (0-15)       : ");
      scanf("%d",&tmp);
      endChan = tmp;
      strtEndChan = strtEndChan | tmp;

      cfgBuf1[18] = strtEndChan; 

      printf("Enter OverSample  (0/1)          : ");
      scanf("%d",&tmp);
      cfgBuf1[19] = tmp; 

      ret =  AIO_Usb_ADC_SetConfig (devIdx,
                                    cfgBuf1,
                                    &cfgBufSize);

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nSetConfig Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }

     break;

   }

   case 13:
   {

     int            i;
     int            ret;
     int            tmp;
     unsigned char  cfgBuf1[20];
     unsigned char  cfgBuf2[20];
     unsigned long  cfgBufSize;


      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      printf("Enter Chan-0 Gain code  : ");
      scanf("%d",&tmp);
      cfgBuf1[0] = tmp; 
      printf("Enter Chan-1 Gain code  : ");
      scanf("%d",&tmp);
      cfgBuf1[1] = tmp; 
      printf("Enter Chan-2 Gain code  : ");
      scanf("%d",&tmp);
      cfgBuf1[2] = tmp; 
      printf("Enter Chan-3 Gain code  : ");
      scanf("%d",&tmp);
      cfgBuf1[3] = tmp; 
      printf("Enter Chan-4 Gain code  : ");
      scanf("%d",&tmp);
      cfgBuf1[4] = tmp; 
      printf("Enter Chan-5 Gain code  : ");
      scanf("%d",&tmp);
      cfgBuf1[5] = tmp; 
      printf("Enter Chan-6 Gain code  : ");
      scanf("%d",&tmp);
      cfgBuf1[6] = tmp; 
      printf("Enter Chan-7 Gain code  : ");
      scanf("%d",&tmp);
      cfgBuf1[7] = tmp; 
      printf("Enter Chan-8 Gain code  : ");
      scanf("%d",&tmp);
      cfgBuf1[8] = tmp; 
      printf("Enter Chan-9 Gain code  : ");
      scanf("%d",&tmp);
      cfgBuf1[9] = tmp; 
      printf("Enter Chan-10 Gain code : ");
      scanf("%d",&tmp);
      cfgBuf1[10] = tmp; 
      printf("Enter Chan-11 Gain code : ");
      scanf("%d",&tmp);
      cfgBuf1[11] = tmp; 
      printf("Enter Chan-12 Gain code : ");
      scanf("%d",&tmp);
      cfgBuf1[12] = tmp; 
      printf("Enter Chan-13 Gain code : ");
      scanf("%d",&tmp);
      cfgBuf1[13] = tmp; 
      printf("Enter Chan-14 Gain code : ");
      scanf("%d",&tmp);
      cfgBuf1[14] = tmp; 
      printf("Enter Chan-15 Gain code : ");
      scanf("%d",&tmp);
      cfgBuf1[15] = tmp; 

      ret =  AIO_Usb_ADC_RangeAll (devIdx,
                                   &cfgBuf1[0]);

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nRangeAll Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }

      ret =  AIO_Usb_ADC_GetConfig (devIdx,
                                    &cfgBuf2[0],
                                    &cfgBufSize); 
     
     if (ret > ERROR_SUCCESS)
     {
      printf ("\nGetConfig after RangeAll Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }
   
     printf("\n\nConguration Read Back: \n"); 
     printf("\t     Note that 8 is added to chan 0-7for differential mode.\n"); 
     for (i=0; i < 16; i++)
     {
       printf("Channel %d Gain Code  = %d\n",i,cfgBuf2[i]);
     } 

     break;

   }

   case 14:
   {

     
     int            ret;
     char           gainCode; 
     unsigned long  chan;
     unsigned long  singleEnded;
     unsigned long  cfgBufSize;
     unsigned char  cfgBuf[20];


      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      printf("Enter Chan to set the Gain code  for : ");
      scanf("%u",(unsigned int *)&chan);

      printf("Enter Gain code                      : ");
      scanf("%u",(unsigned int *)&gainCode);

      printf("Enter '0' for differential mode; '1' for single-ended mode : ");
      scanf("%u",(unsigned int *)&singleEnded);

      ret = AIO_Usb_ADC_Range1 (devIdx,
                                chan,
                                gainCode,
                                singleEnded);      

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nRange1 Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }

      ret =  AIO_Usb_ADC_GetConfig (devIdx,
                                    &cfgBuf[0],
                                    &cfgBufSize); 
     
     if (ret > ERROR_SUCCESS)
     {
      printf ("\nGetConfig after Range1 Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }
   
     printf("Channel %d Gain Code  = %d\n",(unsigned int)chan,cfgBuf[chan]);

     break;

   }
   case 15:
   {

     int            ret;
     unsigned long  trigMode;
     unsigned long  calMode;
     int            tmp;

      ret =  validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;


      printf(" Enter Trigger Mode       : ");
      scanf ("%d",&tmp);
      trigMode = tmp; 


      printf(" Enter Calibration code   : ");
      scanf ("%d",&tmp);
      calMode = tmp;

      ret = AIO_Usb_ADC_ADMode(devIdx,
                               trigMode,
                               calMode);


     if (ret > ERROR_SUCCESS)
     {
      printf ("\nRange1 Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }

     break;
   }
   case 16:
   {
      char           blkIdx; 
      int            ret;
      double         Hz;

      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

      //printf("Enter Block Index                                   :  ");
      //scanf("%d",&tmp);
      //blkIdx = tmp;

      blkIdx = 0; // only 1 for AI16


      printf("Enter Desired Frequency in Hz {1-500,000} <no comma> :  ");
      scanf("%lf",&Hz);

      ret = AIO_CTR_StartOutputFreq(devIdx,
                                    blkIdx,
                                   &Hz);

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nCTR_StartOutputFreq Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }

     printf("\n\n Actual Frequency set to %lf \n",Hz);
     break;
   }

   case 17:
   {
     unsigned long  len;
     int            addr;
     int            i;
     int            ret;
     char           writeBuf[50];
     char           readBuf[50];



     printf ("\n\n Start Adddress {0-0x1FF} (hex) : ");
     scanf ("%x",&addr);

     printf ("\n\n Enter data to write {up to 50 bytes} : ");
     scanf ("%s",(char *)&writeBuf);

     printf ("\n\n Enter data length : ");
     scanf ("%d",(int *)&len);

     ret = validateIndex(devIdx);
     if (ret > ERROR_SUCCESS)
       break;
     
     ret = AIO_Usb_CustomEEPROMWrite(devIdx,
                                     addr,
                                     len,
                                     (void *)writeBuf);
     
     if (ret > ERROR_SUCCESS)
     {
       printf("AIO_ADC_CustomEEProm_Write failed err=%d \n",ret);
     }

     ret = AIO_Usb_CustomEEPROMRead(devIdx,
                                    addr,
                                    len,
                                    (void *)readBuf);
     
     if (ret > ERROR_SUCCESS)
     {
       printf("AIO_ADC_CustomEEProm_Read readback failed after EEPromWrite call err=%d \n",ret);
     }

    for (i=0; i<len; i++)
    {
      if (readBuf[i] != writeBuf[i])
      printf("AIO_ADC_CustomEEProm_Write failed err; data read different then data written =%d \n",ret);
    }
      break;
    }

   case 18:
   {
     unsigned long  len;
     int            addr;
     int            i;
     int            ret;
     char           readBuf[50];



      ret = validateIndex(devIdx);
      if (ret > ERROR_SUCCESS)
       break;

     printf ("\n\n Start Adddress {0-0x1FF} (hex) : ");
     scanf ("%x",&addr);

     printf ("\n\n Enter number of bytes to read {up to 50 bytes}  : ");
     scanf ("%d",(int *)&len);

     ret = AIO_Usb_CustomEEPROMRead(devIdx,
                                    addr,
                                    strlen(readBuf),
                                    (void *)readBuf);
     
     if (ret > ERROR_SUCCESS)
     {
       printf("AIO_ADC_CustomEEProm_Read failed after EEPromRead call err=%d \n",ret);
     }

    for (i=0; i<len; i++)
      printf("  Data read : data[%d] = %c \n",i,readBuf[i]);

   break;
   }



   case 19:
   {
      char           blkIdx; 
      char           counterIdx;
      char           mode;
      int            ret;
      int            tmp;

     //printf ("\n\n Enter Device Idx : ");
     //scanf ("%d",&tmp);
     //devIdx = tmp;

     ret = validateIndex(devIdx);
     if (ret > ERROR_SUCCESS)
     {
       printf("Invalid device. \n");
       break;
     }

      printf("Enter Block Index    :  ");
      scanf("%d",&tmp);
      blkIdx = tmp;
      printf("Enter Counter Index  :  ");
      scanf("%d",&tmp);
      counterIdx = tmp;
      printf("Enter Mode(0-5)      :  ");
      scanf("%d",&tmp);
      mode = tmp;

      ret = AIO_Usb_CTR_8254Mode(devIdx,
                                 blkIdx,
                                 counterIdx,
                                 mode);


     if (ret > ERROR_SUCCESS)
     {
      printf ("\nCTR_8254Mode Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }
     break;
   }
   case 20:
   {
      char           blkIdx; 
      char           counterIdx;
      char           loadVal;
      int            ret;
      int            tmp;

     ret = validateIndex(devIdx);
     if (ret > ERROR_SUCCESS)
     {
       printf("Invalid device. \n");
       break;
     }

      printf("Enter Block Index    :  ");
      scanf("%d",&tmp);
      blkIdx = tmp;
      printf("Enter Counter Index  :  ");
      scanf("%d",&tmp);
      counterIdx = tmp;
      printf("Enter Load Value      :  ");
      scanf("%d",&tmp);
      loadVal  = tmp;

      ret = AIO_Usb_CTR_8254Load(devIdx,
                                 blkIdx,
                                 counterIdx,
                                 loadVal);


     if (ret > ERROR_SUCCESS)
     {
      printf ("\nCTR_8254Load Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }
     break;
   } 
   case 21:
   {
      char           blkIdx; 
      char           counterIdx;
      char           loadVal;
      char           mode;
      int            ret;
      int            tmp;

     ret= validateIndex(devIdx);
     if (ret > ERROR_SUCCESS)
     {
       printf("Invalid device. \n");
       break;
     }

      printf("Enter Block Index    :  ");
      scanf("%d",&tmp);
      blkIdx = tmp;
      printf("Enter Counter Index  :  ");
      scanf("%d",&tmp);
      counterIdx = tmp;
      printf("Enter Mode           :  ");
      scanf("%d",&tmp);
      mode = tmp;
      printf("Enter Load Value     :  ");
      scanf("%d",&tmp);
      loadVal = tmp;
      ret =  AIO_Usb_CTR_8254ModeLoad(devIdx,
                                      blkIdx,
                                      counterIdx,
                                      mode,
                                      loadVal);

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nCTR_8254ModeLoad Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }
     break;
   }
   case 22:
   {
      char           blkIdx; 
      char           counterIdx;
      int            loadVal;
      char           mode;
      int            ret;
      int            tmp;
      unsigned int   counterVal;

     ret = validateIndex(devIdx);
     if (ret > ERROR_SUCCESS)
     {
       printf("Invalid device. \n");
       break;
     }

      printf("Enter Block Index    :  ");
      scanf("%d",&tmp);
      blkIdx = tmp;
      printf("Enter Counter Index  :  ");
      scanf("%d",&tmp);
      counterIdx = tmp;
      printf("Enter Mode           :  ");
      scanf("%d",&tmp);
      mode = tmp;
      printf("Enter Load Value     :  ");
      scanf("%d",&tmp);
      loadVal = tmp;
     ret =  AIO_Usb_CTR_8254ReadModeLoad(devIdx,
                                         blkIdx,
                                         counterIdx,
                                         mode,
                                         loadVal,
                                        &counterVal);

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nCTR_8254ReadModeLoad Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }
     printf("Value of counter = %d \n",counterVal);
     break;
   }



   case 23:
   {
      char           blkIdx; 
      char           counterIdx;
      unsigned int   counterVal;
      int            ret;
      int            tmp;

     ret = validateIndex(devIdx);
     if (ret > ERROR_SUCCESS)
     {
       printf("Invalid device. \n");
       break;
     }

      printf("Enter Block Index    :  ");
      scanf("%d",&tmp);
      blkIdx = tmp;
      printf("Enter Counter Index  :  ");
      scanf("%d",&tmp);
      counterIdx = tmp;

      ret = AIO_Usb_CTR_8254Read(devIdx,
                                 blkIdx,
                                 counterIdx,
                                 &counterVal);

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nCTR_8254Mode Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }
     else
     {
        printf(" Counter Val (decimal) = %d \n",counterVal);
     }
     break;
   }

   case 24:
   {
      int             ret;
      unsigned short  sample;
      double          volts;
 
     ret = validateIndex(devIdx);
     if (ret > ERROR_SUCCESS)
     {
       printf("Invalid device. \n");
       break;
     }

     ret =  AIO_Usb_ADC_GetImmediate(devIdx,
                                    &sample);

     volts = (sample/65535.0) * 10.0;

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nGetImmediate Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }
     else
     {
        printf(" Sample = %f volts \n",volts);
     }

     break;
   }


   case 0: // AtoD samples 
   {

     int            tmp;
     int            i;
     int            j;
     float          volts;
     int            startChan;
     int            endChan;
     int            ret;
     char           calFname[13];
     char           trigger     = 0;
     char           strtEndChan = 0;
     unsigned char  cfgBuf1[20];
     unsigned long  cfgBufSize;
     unsigned char  *dataBuf;
     unsigned long  numSamples = 0;
     size_t         bufSize = 20000000; // 10,000,000 2byte words/samples 
     double         Hz;

     //ret = validateIndex(devIdx);
     //if (ret > ERROR_SUCCESS)
     //  break;


     dataBuf =  (unsigned char *)calloc(bufSize,8); 
     if (dataBuf == NULL)
     {
       printf("AIO_Usb_AtoDSample cannot malloc memory \n");
       break;
     }


     // printf ("\n\n Enter Calibration Filename, or ':NONE:' for default : ");
     // scanf ("%s",calFname);
     // printf ("\n\n");
     strncpy(calFname,":NONE:",6);




      //printf(" Enter number of samples {1-10,000,000} <no comma>: "); // 
      //scanf("%d",&tmp);
      numSamples = 10000;

      strtEndChan = 0;
      endChan     = 15;
      strtEndChan = strtEndChan  | (endChan << 4);

      startChan   = 0;
      strtEndChan = strtEndChan | startChan;
     
 
      cfgBuf1[18] = strtEndChan; 

      // gain codes
      for (i = 0 ; i < 16; i++)
      {
        cfgBuf1[i] = 0; 
      }


      //printf(" Enter CTR0 EXT code (0/1)   : ");
      //scanf("%d",&tmp);
      tmp = 0;
      trigger = trigger | (tmp << 4);
      
      //printf(" Enter Falling Edge (0/1)    : ");
      //scanf("%d",&tmp);
      tmp = 0;
      trigger = trigger | (tmp << 3);


      //printf(" Enter Scan (0/1)            : ");
      //scanf("%d",&tmp);
      tmp = 0;
      trigger = trigger | (tmp << 2);

      //printf(" Enter External Trigger      : ");
      //scanf("%d",&tmp);
      tmp = 0;
      trigger = trigger | (tmp << 1);

      // printf(" Enter Timer Trigger (0/1)   : ");
      //scanf("%d",&tmp);
      tmp = 1;
      trigger = trigger | tmp;

      cfgBuf1[16] = 0; 
      cfgBuf1[17] = trigger; 


      //printf(" Enter OverSample 0/1)       : ");
      //scanf("%d",&tmp);
      tmp = 0;
      cfgBuf1[19] = tmp; 


      ret = AIO_Usb_ADC_Initialize(devIdx,
                                   cfgBuf1,
                                   &cfgBufSize,
                                   calFname);


     if (ret > ERROR_SUCCESS)
     {
      printf ("\nADC_Initialize Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }


    // this function calls issues the USB vendor request
    // to tell the device to start taking A/D samples
    // and sets up the entry in the device_array for the
    // devIdx. this entry is used by the bulkPoll thread
    // to read the data.


    memset((void *)dataBuf,0,sizeof(dataBuf));


    printf("\n\n");

    Hz = 100000;
    
    ret =  AIO_Usb_ADC_BulkAcquire (devIdx,
                                    (unsigned char *)&dataBuf[0],
                                     numSamples,
                                     Hz);
                                     


     if (ret > ERROR_SUCCESS)
     {
      printf ("\nADC_BulkAquire Failed  dev=0x%0x err=%d \n",(unsigned int)devIdx,ret);
      break;
     }

    printf ("\33[2J\n"); // clear screen

    printf("\n\n\n");

    //printf("\33[5;40H Control-C to to quit");
    // catch Control-C
    //signal(SIGINT,catchInterrupt);



    // Bulk Poll is not used in this sample code; it simply takes the samples, waits a short bit
    // assumes they have been taken and displays a snapshot of the samples. Below
    // is an example of how to use the BulkPoll() function to be able to access
    // samples in the buffer before all the requested samples have been taken
 
    //
     // EXAMPLE of how to use BulkPoll
     //


      //printf("\33[14;5HBytes Left  : ");
      //bulkPollTime.tv_sec  = 0; 
      //bulkPollTime.tv_nsec = 2000000000; 

      //bytesLeft = numBytes;
      //while (bytesLeft > 0)
      //{
      //  bytesLeft = AIO_ADC_Usb_BulkPoll(devIdx);
 
        //  The buffer you passed into BulkAcquire above (dataBuf)
        //  contains valid samples up to dataBuf[bytesLeft-1];

        //printf("\33[14;25H %d",bytesLeft);
        //printf("\33[14;5HBytes Left  : ");


        //printf("\33[14;25H %d",bytesLeft);

      //}
    // *************************************
    //  End BulkPoll Example
    // *************************************/


    printf("\n\n");

    printf("\33[8;5H\n ");
    printf ("\33[9;20H===================================================\n"); 
    printf ("\33[10;20H       10,000 Analog to Digital samples           \n");
    printf ("\33[11;20H              Channels 0 - 15                     \n");
    printf ("\33[12;20H              Data Rate = 100,000 Hz              \n");
    printf ("\33[13;20H               Values in Volts                    \n");
    printf ("\33[14;20H              (1st 4 scans only shown)            \n");
    printf ("\33[15;20H===================================================\n"); 


    // column :      9     15             30        40         50         60
    printf("\33[19;9H    Channel        Sample 1   Sample  2   Sample 3   Sample 4 \n");
    printf("\33[20;9H    -------        -------    ---------   --------   --------");
    printf("\33[21;9H      0 ");
    printf("\33[22;9H      1 ");
    printf("\33[23;9H      2 ");
    printf("\33[24;9H      3 ");
    printf("\33[25;9H      4 ");
    printf("\33[26;9H      5 ");
    printf("\33[27;9H      6 ");
    printf("\33[28;9H      7 ");
    printf("\33[29;9H      8 ");
    printf("\33[30;9H      9 ");
    printf("\33[31;9H     10 ");
    printf("\33[32;9H     11 ");
    printf("\33[33;9H     12 ");
    printf("\33[34;9H     13 ");
    printf("\33[35;9H     14 ");
    printf("\33[36;9H     15 ");
    printf("\n\n");


    bulkPollTime.tv_sec  = 0; 
    bulkPollTime.tv_nsec = 300000000; 
 
     j = 0;

     // 
     // The calculation for Volts is shown below :
     //   
     //   dataBuf[i] is the  MSB of the word of the digital count
     //   dataBuf[i+1] is the LSB of the word of the digital count
     //
     //   volts = count word / 65535.0 * (voltage range...set as a Gain Code)
     //   in this example we are using gainCode=0 (0-10 volts)
     //    

   
     // The code below prints 4 samples of each of the 16 channels.
     // The reason it was not done in a loop, but for each sample
     // separately, is each value has to be positioned
     // using the escape for cursor control "\33[x;yH"
     // and x and y cannot be variables but must be 
     // actual numbers


     // Sample 1
     j=0;
     for (i=0; i<16; i++);
     {
      //chan 0
      volts = (unsigned short )dataBuf[j];
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10.0;
      printf("\33[21;30H%f",volts);
      j+=2;
   
 
      // chan 1    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[22;30H%f",volts);
      j+=2;


      // chan 2    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[23;30H%f",volts);
      j+=2;

      // chan 3    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[24;30H%f",volts);
      j+=2;

      // chan 4    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[25;30H%f",volts);
      j+=2;

      // chan 5    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[26;30H%f",volts);
      j+=2;

      // chan 6    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[27;30H%f",volts);
      j+=2;

      // chan 7    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[28;30H%f ",volts);
      j+=2;

      // chan 8    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[29;30H%f",volts);
      j+=2;

      // chan 9    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[30;30H%f",volts);
      j+=2;

      // chan 10    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[31;30H%f",volts);
      j+=2;

      // chan 11    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[32;30H%f",volts);
      j+=2;

      // chan 12    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[33;30H%f",volts);
      j+=2;

      // chan 13    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[34;30H%f",volts);
      j+=2;

      // chan 14    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[35;30H%f",volts);
      j+=2;

      // chan 15
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[36;30H%f",volts);
      j+=2;

    }

    // Sample 2
     j = 32;
     for (i=0; i<16; i++);
     {
      //chan 0
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[21;40H%f",volts);
      j+=2;
   
 
      // chan 1    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[22;40H%f",volts);
      j+=2;


      // chan 2    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[23;40H%f",volts);
      j+=2;

      // chan 3    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[24;40H%f",volts);
      j+=2;

      // chan 4    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[25;40H%f",volts);
      j+=2;

      // chan 5    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[26;40H%f",volts);
      j+=2;

      // chan 6    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[27;40H%f",volts);
      j+=2;

      // chan 7    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[28;40H%f",volts);
      j+=2;

      // chan 8    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[29;40H%f",volts);
      j+=2;

      // chan 9    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[30;40H%f",volts);
      j+=2;

      // chan 10    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[31;40H%f",volts);
      j+=2;

      // chan 11    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[32;40H%f",volts);
      j+=2;

      // chan 12    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[33;40H%f",volts);
      j+=2;

      // chan 13    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[34;40H%f",volts);
      j+=2;

      // chan 14    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[35;40H%f",volts);
      j+=2;

      // chan 15
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[36;40H%f",volts);
      j+=2;

    }
 
    // Sample 3
    j = 64;
     for (i=0; i<16; i++);
     {
      //chan 0
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[21;50H%f",volts);
      j+=2;
   
 
      // chan 1    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[22;50H%f",volts);
      j+=2;


      // chan 2    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[23;50H%f",volts);
      j+=2;

      // chan 3    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[24;50H%f",volts);
      j+=2;

      // chan 4    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[25;50H%f",volts);
      j+=2;

      // chan 5    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[26;50H%f",volts);
      j+=2;

      // chan 6    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[27;50H%f",volts);
      j+=2;

      // chan 7    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[28;50H%f",volts);
      j+=2;

      // chan 8    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[29;50H%f",volts);
      j+=2;

      // chan 9    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[30;50H%f",volts);
      j+=2;

      // chan 10    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[31;50H%f",volts);
      j+=2;

      // chan 11    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[32;50H%f",volts);
      j+=2;

      // chan 12    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[33;50H%f",volts);
      j+=2;

      // chan 13    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[34;50H%f",volts);
      j+=2;

      // chan 14    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[35;50H%f",volts);
      j+=2;

      // chan 15
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[36;50H%f",volts);
      j+=2;

    }

    // Sample 4
    j = 96;
     for (i=0; i<16; i++);
     {
      //chan 0
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[21;60H%f",volts);
      j+=2;
   
 
      // chan 1    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[22;60H%f",volts);
      j+=2;


      // chan 2    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[23;60H%f",volts);
      j+=2;

      // chan 3    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[24;60H%f",volts);
      j+=2;

      // chan 4    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[25;60H%f",volts);
      j+=2;

      // chan 5    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[26;60H%f",volts);
      j+=2;

      // chan 6    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[27;60H%f",volts);
      j+=2;

      // chan 7    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      volts = (volts/65535.0) * 10;
      printf("\33[28;60H%f",volts);
      j+=2;

      // chan 8    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[29;60H%f",volts);
      j+=2;

      // chan 9    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[30;60H%f",volts);
      j+=2;

      // chan 10    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[31;60H%f",volts);
      j+=2;

      // chan 11    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[32;60H%f",volts);
      j+=2;

      // chan 12    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[33;60H%f",volts);
      j+=2;

      // chan 13    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[34;60H%f",volts);
      j+=2;

      // chan 14    
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[35;60H%f",volts);
      j+=2;

      // chan 15
      volts = (dataBuf[j+1] << 8) | (dataBuf[j]);
      volts = (volts/65535.0) * 10;
      printf("\33[36;60H%f",volts);
      j+=2;

    }
   free(dataBuf);

   } // case 0  AtoD Samples



    default :
    break;
   } // switch


  } // while 

  } // main 
