// 
// Sample of a Digital Input Program for ACCES/IO DIO-16 using the fast-DIO bus 
//
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include "libusb.h"

#include "aioUsbApi.h"

aioDeviceInfo aioDevices;

int main()
{
  int            fd;
  unsigned long  bytesStreamed;
  int            fileLen;
  char           input;
  int            i; 
  int            devIdx; 
  int            devFound; 
  int            ret;
  int            timing;
  unsigned char  triState;
  double         readHz;
  double         writeHz;
  unsigned char  mask;
  unsigned char  data[4];
  char           dataFname[50];

  unsigned short *fileBuf;


    printf ("\33[2J\n"); // clear screen

    printf("\33[8;5H\n ");
    printf ("\33[9;20H===================================================\n"); 
    printf ("\33[10;20H       ACCES/IO Digital Input Sample Program     \n"); 
    printf ("\33[11;20H              USB-DIO-16* Product Family          \n");
    printf ("\33[12;20H===================================================\n"); 

     printf("\n\n");
     printf("                 IMPORTANT : Please read the  note below, or you can\n");
     printf("                             damage your product !!!             \n\n"); 
     printf("            Be certain that you are NOT connecting signals that will cause\n");
     printf("            bus contention (outputs to outputs) before you proceed !\n\n");

     printf("\n\n");
     printf("             This sample receives data from the input ports of the DIO-16 device       \n"); 
     printf("             and saves it to disk.\n"); 
     printf("\n\n");
     printf("             The program does not allow you to configure the direction of the various      \n");
     printf("             slow-mode digital I/O ports, and dedicates the fast-DIO bus to *input* mode     \n");
     printf("             which causes certain hand-shaking signals to become outputs                     \n"); 
     printf("\n\n\n\n");

     printf("            <<Enter the ANY key to continue, or Control-C to ABORT>> : "); 
     scanf("%s",&input);
     printf("\n\n");

     AIO_Init();
     ret = AIO_Usb_GetDevices(&aioDevices);
     if (ret > ERROR_SUCCESS)
     {
       printf("AIO_Usb_GetDevices failed! (be sure there is an Acces/IO device connected to your computer) ret=%d \n",ret);
       return 0;
     }


     // find the first DIO-16 device
     devFound = 0;
     for (i = 0; i < MAX_USB_DEVICES;i++)
     {
       if ( (aioDevices.aioDevList[i].productId == USB_DIO_16H) ||
            (aioDevices.aioDevList[i].productId == USB_DI_16A)  ||
            (aioDevices.aioDevList[i].productId == USB_DO_16A)  ||
            (aioDevices.aioDevList[i].productId == USB_DIO_16A)    )
       {
         devIdx = aioDevices.aioDevList[i].devIdx;
         devFound = 1; 
       }
     }

     if ( devFound == 0)
     {
       printf("No DIO_16, or DI_16 devices found \n");
       return 0;
     } 

     AIO_UsbListDevices();
   
     printf ("\n\n Enter Device Idx : ");
     scanf ("%d",(unsigned int *)&devIdx);
  
     ret = AIO_UsbValidateDeviceIndex(devIdx);
     if (ret > ERROR_SUCCESS)
     {
       printf("\n\nDBG>> Invalid Device Index \n"); 
       return 0;
     }

      printf("\n\tSelect Internal or External Timing (0=INternal; 1=EXternal)      : "); 
      scanf("%d",&timing);

      if ( (timing != 0) && (timing !=1) )
      {
        printf("\tInvalid selection (%d). Must be 0 or 1 \n",timing);
        return 0;
      }

      // only need to set read clock for this digital output sample
      writeHz= 0;

      if (timing == 1)
      {
        readHz = 0;  // disable the write clock since it will be externally provided
      }
      else if (timing == 0)
      {
        printf("\tEnter Desired Write Clock Frequency {1,000-8,000,000} <no comma> : ");
        scanf("%lf",&readHz);

        
        if ( (readHz < 1000) || (readHz > 8000000) )
        {
           printf("\n\tInvalid selection (%f). Must be between 1,000 and 8,000,000 \n",writeHz);
           return 0;
        }
      }

     // StreamOpen MUST be called before ANY other call 
     // or the word will come to an end
     ret =   AIO_Usb_DIO_StreamOpen(devIdx,STREAM_OP_READ);  // 0 sets stream for writing
     if (ret > ERROR_SUCCESS)
     {
      printf ("\n\tDIO_StreamOpenFailed  dev=%d err=%d \n",(unsigned int)devIdx,ret);
      return 0;
     }

      // set slow DIO ports A,B,C to triState 
      // since these slow ports are not used for this
      // program. Only the fast DIO bus is used
      //        
      // The least significant bit tristates Port-A
      // The next least significant bit tristates Port B,C,D
      // 
      triState = 0x03;

      // set all ports to outputs 
      // but does not matter because they
      // are tri-stated
      mask = 0x3;

      // data to be output on ouput pins
      // but does not matter since they 
      // are tristated for this sample program
      data[0] = 0;
      data[1] = 0;
      data[2] = 0;
      data[3] = 0;


      ret =   AIO_Usb_DIO_ConfigureEx(devIdx,
                                      &mask,
                                       data,   
                                       triState);

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nDIO_Configure Failed  dev=%d err=%d \n",(unsigned int)devIdx,ret);
      return (0); 
     }

     printf ("\n\tEnter Name of File to Save to Disk : "); 
     scanf ("%s",dataFname);

     printf ("\n\tEnter Length (decimal) of file in Bytes (2 bytes = 1 FramePoint): \n"); 
     printf ("\n\t Length MUST be a multiple of (256 2-byte samples) - 512 bytes : "); 
     scanf ("%d",&fileLen);

     if ( (fileLen % 256 != 0))
     {
       printf ("\n\t file length (%d) not a multiple of 256 , or less than 1024 \n",fileLen);
       return 0;
     }

     fileBuf = (unsigned short *)calloc(fileLen,1);
     if (fileBuf == NULL)
     {
      printf ("\n\tCannot allocate memory \n");
      return 0;
     }
 
     fd = creat(dataFname,S_IWUSR);
     if (fd < 0)
     {
       printf ("\n\tCannot create file : %s \n",dataFname);
       return (0); 
     }


     printf("\n\tData is being saved to file : %s \n",dataFname);
 

        bytesStreamed = 0;        
        ret = AIO_Usb_DIO_StreamFrame(devIdx,
                                      fileLen/2, // function takes framePoints as arg which is 16bits
                                      fileBuf,
                                     &bytesStreamed,
                                     &readHz);
       if (ret > ERROR_SUCCESS)
       {
         printf ("\nDIO_StreamFrame Failed  dev=%d err=%d \n",(unsigned int)devIdx,ret);
         printf ("\n  ONLY %d bytes streamed \n",(unsigned int)bytesStreamed); 
         return 0;
       }
/***
      if ( (bytesStreamed ) != (fileLen) )
      {
        printf ("\nDIO_StreamFrame Failed  dev=%d err=%d \n",(unsigned int)devIdx,ret);
        printf ("\n  ONLY %d bytes streamed successfully\n",(unsigned int)bytesStreamed);
        return 0;
      }
***/

    printf("\n\n\t%d TOTAL Bytes Successfully Received \n",(unsigned int)fileLen);

    ret = write (fd,(void *)fileBuf,fileLen);

    if (ret <= 0)
    {    
      printf("\n  Could not write file\n");
      return 0;
    } 

    close(fd); 


     ret =  AIO_Usb_DIO_StreamClose(devIdx);
     if (ret > ERROR_SUCCESS)
     {
      printf ("\nDIO_StreamClose Failed  dev=%d err=%d \n",(unsigned int)devIdx,ret);
      return 0;
     }

 return 0;

}
