 /*
  *  Sample of a Digital Output Program for ACCES/IO DIO-16 using the fast-DIO bus 
  *  
  */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include "libusb.h"

#include "aioUsbApi.h"

aioDeviceInfo aioDevices;

int main()
{

 FILE          *file; 
  int            ret;
  int            fd;
  int            framePoints;
  unsigned long  bytesStreamed;
  int            numBytes;
  char           input;
  int            i; 
  int            devIdx; 
  int            devFound; 
  int            timing;
  unsigned char  triState;
  double         readHz;
  double         writeHz;
  unsigned char  mask;
  unsigned char  data[4];
  char           dataFname[50];
  int 		 count;
  unsigned char *fileBuf;
  div_t          bufSize;
  int            fileLen;

  //unsigned short fileBuf[5000000];


    printf ("\33[2J\n"); // clear screen

    printf("\33[8;5H\n ");
    printf ("\33[9;20H===================================================\n"); 
    printf ("\33[10;20H       ACCES/IO Digital Output Sample Program     \n"); 
    printf ("\33[11;20H              USB-DIO-16* Product Family          \n");
    printf ("\33[12;20H===================================================\n"); 

     printf("\n\n");
     printf("                 IMPORTANT : Please read the  note below, or you can\n");
     printf("                             damage your product !!!             \n\n"); 
     printf("            Be certain that you are NOT connecting signals that will cause\n");
     printf("            bus contention (outputs to outputs) before you proceed !\n\n");

     printf("\n\n");
     printf("             This sample sends a file to the output ports of the DIO-16 device.       \n"); 
     printf("             If the length of the file is an odd number of bytes, a '0' will be appended to it\n"); 
     printf("             so there an even number of whole FramePoints(2 bytes each) sent.                  \n"); 
     printf("\n\n");
     printf("             The program does not allow you to configure the direction of the various      \n");
     printf("             slow-mode digital I/O ports, and dedicates the fast-DIO bus to *output* mode     \n");
     printf("\n\n\n\n");

     printf("            <<Enter the ANY key to continue, or Control-C to ABORT>> : "); 
     scanf("%s",&input);
     printf("\n\n");

     AIO_Init();
     ret = AIO_Usb_GetDevices(&aioDevices);
     if (ret > ERROR_SUCCESS)
     {
       printf("AIO_Usb_GetDevices failed! (be sure there is an Acces/IO device connected to your computer) ret=%d \n",ret);
       return 0;
     }


     devFound  = 0;
     for (i = 0; i < MAX_USB_DEVICES;i++)
     {
       if ( (aioDevices.aioDevList[i].productId == USB_DIO_16H) ||
            (aioDevices.aioDevList[i].productId == USB_DI_16A)  ||
            (aioDevices.aioDevList[i].productId == USB_DO_16A)  ||
            (aioDevices.aioDevList[i].productId == USB_DIO_16A)    )
       {
         devIdx = aioDevices.aioDevList[i].devIdx;
         devFound = 1; 
       }
     }

     if ( devFound == 0)
     {
       printf("No DI_16, DIO_16 or DO_16 devices found \n");
       return 0;
     } 
     AIO_UsbListDevices();

     printf ("\n\n Enter Device Idx : ");
     scanf ("%d",(unsigned int *)&devIdx);
  
     ret = AIO_UsbValidateDeviceIndex(devIdx);
     if (ret > ERROR_SUCCESS)
     {
       printf("\n\nDBG>> Invalid Device Index \n"); 
       return 0;
     }

      printf("\n\tSelect Internal or External Timing (0=INternal; 1=EXternal)      : "); 
      scanf("%d",&timing);

      if ( (timing != 0) && (timing !=1) )
      {
        printf("\tInvalid selection (%d). Must be 0 or 1 \n",timing);
        return 0;
      }

      // only need to set write clock for this digital output sample
      readHz  = 0;

      if (timing == 1)
      {
        writeHz = 0;  // disable the write clock since it will be externally provided
      }
      else if (timing == 0)
      {
        printf("\tEnter Desired Write Clock Frequency {1,000-8,000,000} <no comma> : ");
        scanf("%lf",&writeHz);
        
        if ( ( writeHz < 1000) || (writeHz > 8000000) )
        {
           printf("\n\tInvalid selection (%f). Must be between 1,000 and 8,000,000 \n",writeHz);
           return 0;
        }
      }


     // StreamOpen MUST be called before anyother CALL to this API 
     // or the word will come to an end

     ret =   AIO_Usb_DIO_StreamOpen(devIdx,STREAM_OP_WRITE);  // 0 sets stream for writing
     if (ret > ERROR_SUCCESS)
     {
      printf ("\n\tDIO_StreamOpenFailed  dev=%d err=%d \n",(unsigned int)devIdx,ret);
      return 0;
     }

      // set slow DIO ports A,B,C to triState 
      // since these slow ports are not used for this
      // program. Only the fast DIO bus is used
      //        
      // 
      // The least significant bit tristates Port-A
      // The next least significant bit tristates Port B,C,D
      // 
      triState = 0x03;

      // set all ports to outputs 
      // but does not matter because they
      // are tri-stated
      mask = 0x3;

      // data to be output on ouput pins
      // but does not matter since they 
      // are tristated for this sample program
      data[0] = 0;
      data[1] = 0;
      data[2] = 0;
      data[3] = 0;


      ret =   AIO_Usb_DIO_ConfigureEx(devIdx,
                                      &mask,
                                       data,   
                                       triState);

     if (ret > ERROR_SUCCESS)
     {
      printf ("\nDIO_Configure Failed  dev=%d err=%d \n",(unsigned int)devIdx,ret);
      return (0); 
     }

     printf ("\n\tEnter Name of File to send to Output ports :");
     scanf ("%s",dataFname);


    file = fopen(dataFname,"r");
    if (file == NULL)
    {
      printf ("\n\tCannot open file : %s \n",dataFname);
      return (0); 
    }


   fseek(file,0L,SEEK_END);
   numBytes = ftell(file); 


 
   fclose(file); 

   fd = open(dataFname,O_RDONLY);

   bufSize.quot = 0;
   bufSize.rem  = 0;
   fileLen      = numBytes;
 

     printf ("\n\t Input File length = %d  bytes. Sending file of length %d bytes. \n", numBytes,fileLen);
     printf("\n\tPress any key to send file : ");
     scanf("%s",&input);
     printf("\n\n");


    fileBuf  =  (unsigned char *)calloc(fileLen,1); 
    memset(fileBuf,'\0',fileLen);
    if (fileBuf == NULL)
    {
        printf("\n\t Cannot alloc of (%d) bytes of memory for file %s \n",fileLen,dataFname);
        return 0;
     }
   

    count = read(fd,fileBuf,numBytes);
    close(fd); 

     printf("\n\t Writing file (%s) of (%d) bytes to output ports \n",dataFname,numBytes);



      framePoints = fileLen / 2;  // 1 framepoint is 2 bytes

      bytesStreamed = 0;

      ret = AIO_Usb_DIO_StreamFrame(devIdx,
                                    framePoints, 
                                    fileBuf,
                                   &bytesStreamed,
                                   &writeHz);
       if (ret > ERROR_SUCCESS)
       {
          printf ("\nDIO_StreamFrame Failed  dev=%d err=%d \n",(unsigned int)devIdx,ret);
          printf ("\n  ONLY %d bytes of %d total bytes streamed \n",(unsigned int)bytesStreamed,fileLen);
          return 0;
      }
      else
      {
          printf ("\n\t.... %d Total bytes successfully streamed \n",(unsigned int)bytesStreamed);
      }


     ret =   AIO_Usb_DIO_StreamClose(devIdx);
     if (ret > ERROR_SUCCESS)
     {
      printf ("\nDIO_StreamClose Failed  dev=%d err=%d \n",(unsigned int)devIdx,ret);
      return 0;
     }

    return 0;

}
