#!/bin/sh 

# This script loads the Firmware into an ACCES/IO USB Device
# using Linux's 'udev' hot plugging feature
#
# The script is called when an ACCES/IO USB device is connected/plugged in. The
# file /etc/udev/rules.s/acces.rules contains the rule which causes
# the script to run.
# As can be seen in the rule, when the rule is invoked it sets 2 ENVironment
# variables ($BUS, $DEVICE). 
#
# $BUS is set in /etc/udev/rules.d/acces_usb.rules and contains the usb bus number which
# will be "/dev/bus/usb/xxx", where xxx is the specific bus number
# chosen by Linux (e.g. 002).
# (the bus number can change each time a device is connected)

# $DEV is set in /etc/udev/rules.d/acces_usb.rules and contains the usb device number 
# will be a number assigned to the device (e.g. "9", "22", "100").
# This number changes each time the device is connected to the system.
# (it foes from 001 to 0099 then wraps back to 001) 
# 
# The bus numbers are found in both
# /dev/bus/usb/xxx/yyy  (xxx is the bus, and yyy is the device)
# /proc/bus/usb/xxx/yyy  (xxx is the bus, and yyy is the device)
#

# some shells on some systems vary, so the syntax here may need to be
# adjusted accordingly but the logic should not be changed
# 

if [ $1 == "--load" ]
then

case $2 in
   "--0040") hexfile="/usr/share/usb/USB-AI16-16.hex";;
   "--0041") hexfile="/usr/share/usb/USB-AI16-16.hex";;
   "--0042") hexfile="/usr/share/usb/USB-AI16-16.hex";;
   "--0043") hexfile="/usr/share/usb/USB-AI16-16.hex";;
   "--0044") hexfile="/usr/share/usb/USB-AI16-16.hex";;
   "--0045") hexfile="/usr/share/usb/USB-AI16-16.hex";;

   "--000c") hexfile="/usr/share/usb/USB-DIO-16H-A.hex";;
   "--000e") hexfile="/usr/share/usb/USB-DIO-16H-A.hex";;
   "--000d") hexfile="/usr/share/usb/USB-DIO-16H-A.hex";;
   "--000f") hexfile="/usr/share/usb/USB-DIO-16H-A.hex";;

   "--0001") hexfile="/usr/share/usb/USB-DIO-32.hex";;

   "--0010") hexfile="/usr/share/usb/USB-IIRO-16.hex";;
   "--0018") hexfile="/usr/share/usb/USB-IDIO-16.hex";;
   *) hexfile="invalid";;
esac


 if [ $DEV -lt 10 ]  
 then
    DEVICE = $BUS/00$DEV
 elif [ test $DEV -lt 99 ] 
 then
  DEVICE = $BUS/0$DEV
 else
    DEVICE = $BUS/$DEV
  fi

  echo `date` >> /usr/share/usb/AccesDevice 
  echo "ACCES IO Device loaded was : "  $DEVICE  >> /usr/share/usb/AccesDevice

 echo "attempting to fxload firmware"
 `/sbin/fxload -t fx2 -D $DEVICE -I $hexfile`
fi



if [ $1 == "--chmod" ]
then
    chmod 0666 $DEVICE
fi
