/*
 *   This just tests all the functions of the driver from user-mode.
 *   Run this in an ANSI terminal emulation, i.e., xterm or Linux
 *   console/terminal because it uses ANSI escape sequences for
 *   cursor addressing.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <sys/poll.h>
#include <sys/time.h>
#include "adcboard.h"
#define FAIL -1
static const char adc[]="ADC";
static const char esc[]={27, '[', 'K', 27, '[', 0};
static const char put[]="%s%d;10HChannel %-2d = %d";
static const char era[]={27, '[', 'H', 27, '[', 'J', 27, '[',\
                        '6', ';', '6', 'H', 0};
int main(void);
int main()
{
    struct timeval tv;
    int buf[0x1000];
    struct adc_list point;
    int *ip;
    fd_set rfds;
    struct adc_multi dat;
    int fd, tmp, nr, i;
    (void)umask(0);
    if((fd = open(adc, O_RDWR, 0)) == FAIL)
        (void)mknod(adc, 0666|S_IFCHR, DEV_MAJOR << 0x08);
            if((fd = open(adc, O_RDWR, 0)) == FAIL)
            {
                fprintf(stderr, "Can't open device %s\n", adc);
                if(errno == ENOENT)
                {
                    fprintf(stderr, "You need to be root for this test\n");
                    fprintf(stderr, ".... (at least the first time)!\n");
                }
                else
                    fprintf(stderr, "Maybe `insmod ioadc.o` ?\n");
                exit(EXIT_FAILURE);
            }
    tmp = 100;      /* Microseconds to set the timer */
/*
 *  Since the actual frequency input to the dividers is 1 MHz, the
 *  divisor is is microseconds. I don't suggest using less than 10
 *  microseconds, the MUX may not work correctly and the converter
 *  may not properly convert.
 */
    if(ioctl(fd, ADC_SET_TIMER, &tmp) == FAIL)
    {
        fprintf(stderr, "Can't set the timer\n");
        exit(EXIT_FAILURE);
    }
/*
 *  I'm going to set all possible input configurations. The header file,
 *  adcboard.h shows what they mean.
 */
    if(ioctl(fd, ADC_BIP_10, NULL) == FAIL)
    {
        fprintf(stderr, "Can't set the input mode\n");
        exit(EXIT_FAILURE);
    }
    if(ioctl(fd, ADC_BIP_05, NULL) == FAIL)
    {
        fprintf(stderr, "Can't set the input mode\n");
        exit(EXIT_FAILURE);
    }
    if(ioctl(fd, ADC_BIP_2d5, NULL) == FAIL)
    {
        fprintf(stderr, "Can't set the input mode\n");
        exit(EXIT_FAILURE);
    }
    if(ioctl(fd, ADC_BIP_1d25, NULL) == FAIL)
    {
        fprintf(stderr, "Can't set the input mode\n");
        exit(EXIT_FAILURE);
    }
    if(ioctl(fd, ADC_UNI_10, NULL) == FAIL)
    {
        fprintf(stderr, "Can't set the input mode\n");
        exit(EXIT_FAILURE);
    }
    if(ioctl(fd, ADC_UNI_05, NULL) == FAIL)
    {
        fprintf(stderr, "Can't set the input mode\n");
        exit(EXIT_FAILURE);
    }
    if(ioctl(fd, ADC_UNI_1d25, NULL) == FAIL)
    {
        fprintf(stderr, "Can't set the input mode\n");
        exit(EXIT_FAILURE);
    }
    if(ioctl(fd, ADC_UNI_6d25, NULL) == FAIL)
    {
        fprintf(stderr, "Can't set the input mode\n");
        exit(EXIT_FAILURE);
    }
    if(ioctl(fd, ADC_BIP_1d25, NULL) == FAIL)
    {
        fprintf(stderr, "Can't set the input mode\n");
        exit(EXIT_FAILURE);
    }
    tmp = 20;        /* Filter coeficient  */
    if(ioctl(fd, ADC_SET_FILTER, &tmp) == FAIL)
    {
        fprintf(stderr, "Can't set the filter\n");
        exit(EXIT_FAILURE);
    }
    if(ioctl(fd, ADC_SINGLE_CHAN, NULL) == FAIL)
    {
        fprintf(stderr, "Can't start data acquisition\n");
        exit(EXIT_FAILURE);
    }
/*
 *  First, just slowly read the values in all the channels.
 */
    ip = (int *) buf;
    for(i=0; i< 0x10; i++)
    {
        if(ioctl(fd, ADC_SET_CHAN, &i) == FAIL)
        {
            fprintf(stderr, "Can't set input channel\n");
            exit(EXIT_FAILURE);
        }
        tv.tv_sec  = 1;
        tv.tv_usec = 0;
        FD_ZERO(&rfds);
        FD_SET(fd,  &rfds);
        select(fd+1, &rfds, NULL, NULL, &tv);
        if(FD_ISSET(fd, &rfds))
        {
            if((nr = read(fd, buf, sizeof(buf))) > 0)
                printf("Read %d integer from channel %d, value was %d\n",
                     nr/sizeof(int), i, *ip);
            else
                printf("Read nothing from channel %d\n", i);
        }
    }
    printf("Hit [Enter] to continue...");
    fflush(stdout);
    (void)read(STDIN_FILENO, buf, sizeof(buf));
/*
 *  Now do multi-channel data acquisition in a loop until terminated
 *  by hitting [Enter].
 */
    tmp = 100;      /* Microseconds to set the timer */
    if(ioctl(fd, ADC_SET_TIMER, &tmp) == FAIL)
    {
        fprintf(stderr, "Can't set the timer\n");
        exit(EXIT_FAILURE);
    }
/*
 *  The IIR filter coeficient is best set experimentally. If it's
 *  a low number, the values are observed to change instantly, however
 *  noise may make the values unusable in the real world. If the
 *  coefficient is set too high, the values will change too slowly
 *  to be useful in some applications. The time-constant of the
 *  resulting 2-poll filter is TC = ADC_TIMER * ADC_SET_FILTER
 *  * 1.58, where ADC_TIMER is in microseconds. So with a timer
 *  of 100 us and a filter of 20 the time-constant is 100 * 20 *
 *  1.58 =  3160 microseconds.
 */
    tmp = 20;        /* Filter coeficient  */
    if(ioctl(fd, ADC_SET_FILTER, &tmp) == FAIL)
    {
        fprintf(stderr, "Can't set the filter\n");
        exit(EXIT_FAILURE);
    }
    if(ioctl(fd, ADC_MULTI_CHAN, NULL) == FAIL)
    {
        fprintf(stderr, "Can't start data acquisition\n");
        exit(EXIT_FAILURE);
    }
    printf(era);
    fflush(stdout);
    printf("Hit [Enter] to exit this loop.\n");
    fflush(stdout);
/*
 *  Here we will read the values of all the A/D channels. Reading
 *  them does not upset them in any way. The driver is acquiring
 *  and filtering the A/D channels continuously with minimal
 *  impact upon your computer. When you want to read a value,
 *  just read the structure containing them all. It's that simple.
 */
    for(;;)
    {
        tv.tv_sec  = 0;
        tv.tv_usec = 10000;
        FD_ZERO(&rfds);
        FD_SET(STDIN_FILENO,  &rfds);
        select(STDIN_FILENO+1, &rfds, NULL, NULL, &tv);
        if(FD_ISSET(STDIN_FILENO, &rfds))
            break;
        if(ioctl(fd, ADC_READ_MULTI, &dat) == FAIL)
        {
            fprintf(stderr, "Can't read the data\n");
            exit(EXIT_FAILURE);
        }
        ip = (int *) &dat;
        for(i=0; i< 0x10; i++)
             printf(put, esc, i + 8, i, *ip++);
        fflush(stdout); 
    }
    fflush(stdout);
    (void)read(STDIN_FILENO, buf, sizeof(buf));
/*
 *  Now I will reconfigure all the channels for the point-list.
 *  This should fail if the board does not have a FIFO.
 *  Of course you can fill this thing in with a loop. But
 *  it must be completely filled in. 
 */
    memset(&point, 0x00, sizeof(point));
    point.list[0x00].input = ADC_R0|ADC_R1;
    point.list[0x01].input = ADC_R0|ADC_R1;
    point.list[0x02].input = ADC_R0|ADC_R1;
    point.list[0x03].input = ADC_R0|ADC_R1;
    point.list[0x04].input = ADC_R0|ADC_R1;
    point.list[0x05].input = ADC_R0|ADC_R1;
    point.list[0x06].input = ADC_R0|ADC_R1;
    point.list[0x07].input = ADC_R0|ADC_R1;
    point.list[0x08].input = ADC_R0|ADC_R1;
    point.list[0x08].input = ADC_R0|ADC_R1;
    point.list[0x09].input = ADC_R0|ADC_R1;
    point.list[0x0a].input = ADC_R0|ADC_R1;
    point.list[0x0b].input = ADC_R0|ADC_R1;
    point.list[0x0c].input = ADC_R0|ADC_R1;
    point.list[0x0d].input = ADC_R0|ADC_R1;
    point.list[0x0e].input = ADC_R0|ADC_R1;
    point.list[0x0f].input = ADC_R0|ADC_R1;
    point.list[0x00].filter = 100;
    point.list[0x01].filter = 100;
    point.list[0x02].filter = 100000;   /* So it shows  channel 2 */
    point.list[0x03].filter = 100;
    point.list[0x04].filter = 100;
    point.list[0x05].filter = 100;
    point.list[0x06].filter = 100;
    point.list[0x07].filter = 100;
    point.list[0x08].filter = 100;
    point.list[0x09].filter = 100;
    point.list[0x0a].filter = 100000;  /* So it shows  channel 10 */
    point.list[0x0b].filter = 100;
    point.list[0x0c].filter = 100;
    point.list[0x0d].filter = 100;
    point.list[0x0e].filter = 100;
    point.list[0x0f].filter = 100;
    if(ioctl(fd, ADC_POINT_LIST, &point) == FAIL)
    {
        fprintf(stderr, "Can't set the point-list data\n");
        (void)sleep(1);
    }

    for(;;)
    {
        tv.tv_sec  = 0;
        tv.tv_usec = 10000;
        FD_ZERO(&rfds);
        FD_SET(STDIN_FILENO,  &rfds);
        select(STDIN_FILENO+1, &rfds, NULL, NULL, &tv);
        if(FD_ISSET(STDIN_FILENO, &rfds))
            break;
        if(ioctl(fd, ADC_READ_MULTI, &dat) == FAIL)
        {
            fprintf(stderr, "Can't read the data\n");
            exit(EXIT_FAILURE);
        }
        ip = (int *) &dat;
        for(i=0; i< 0x10; i++)
             printf(put, esc, i + 8, i, *ip++);
        fflush(stdout); 
    }
    fflush(stdout);
    (void)read(STDIN_FILENO, buf, sizeof(buf));

    if(ioctl(fd, ADC_SINGLE_CHAN, NULL) == FAIL)
    {
        fprintf(stderr, "Can't start data acquisition\n");
        exit(EXIT_FAILURE);
    }
    i = 7;
    if(ioctl(fd, ADC_SET_CHAN, &i) == FAIL)
    {
        fprintf(stderr, "Can't set input channel\n");
        exit(EXIT_FAILURE);
    }
    printf("Hit [Enter] to exit this loop.\n");
    fflush(stdout);
    for(;;)
    {
        usleep(1000);
        FD_ZERO(&rfds);
        FD_SET(fd, &rfds);
        FD_SET(STDIN_FILENO,  &rfds);
        select(fd+1, &rfds, NULL, NULL, NULL);
        if(FD_ISSET(fd, &rfds))
        {
            if((nr = read(fd, buf, sizeof(buf))) > 0)
            {
                for(i=0; i< (nr/sizeof(int)); i++)
                {
                    printf("\t%08x  \r", buf[i]);
                    fflush(stdout);
                }
            }
        }
        if(FD_ISSET(STDIN_FILENO, &rfds))
            break;
    }
    printf("\n");
    (void)read(STDIN_FILENO, buf, sizeof(buf));
    if(ioctl(fd, ADC_GET_INTRS, &i) == FAIL)
    {
        fprintf(stderr, "Can't read interrupt count.\n");
        exit(EXIT_FAILURE);
    }
    printf("Driver interrupt count is %u\n", i);
    (void)close(fd);
    return 0;
}


